@@ -1,6 +1,9 @@
 /*************************************************************************************
  *  Copyright (C) 2012 by Alejandro Fiestas Olivares <afiestas@kde.org>              *
  *  Copyright 2016 by Sebastian Kügler <sebas@kde.org>                               *
+ *  Copyright (c) 2018 Kai Uwe Broulik <kde@broulik.de>                              *
+ *                    Work sponsored by the LiMux project of                         *
+ *                    the city of Munich.                                            *
  *                                                                                   *
  *  This program is free software; you can redistribute it and/or                    *
  *  modify it under the terms of the GNU General Public License                      *
@@ -51,7 +54,7 @@ KScreenDaemon::KScreenDaemon(QObject* parent, const QList< QVariant >& )
  , m_monitoredConfig(nullptr)
  , m_monitoring(false)
  , m_changeCompressor(new QTimer(this))
- , m_saveTimer(new QTimer(this))
+ , m_saveTimer(nullptr)
  , m_lidClosedTimer(new QTimer(this))
  
 {
@@ -91,15 +94,11 @@ void KScreenDaemon::init()
     action->setText(i18n("Switch Display" ));
     QList<QKeySequence> switchDisplayShortcuts({Qt::Key_Display, Qt::MetaModifier + Qt::Key_P});
     KGlobalAccel::self()->setGlobalShortcut(action, switchDisplayShortcuts);
-    connect(action, &QAction::triggered, [&](bool) { displayButton(); });
+    connect(action, &QAction::triggered, this, &KScreenDaemon::displayButton);
 
     new KScreenAdaptor(this);
     // Initialize OSD manager to register its dbus interface
-    KScreen::OsdManager::self();
-
-    m_saveTimer->setInterval(300);
-    m_saveTimer->setSingleShot(true);
-    connect(m_saveTimer, &QTimer::timeout, this, &KScreenDaemon::saveCurrentConfig);
+    m_osdManager = new KScreen::OsdManager(this);
 
     m_changeCompressor->setInterval(10);
     m_changeCompressor->setSingleShot(true);
@@ -113,7 +112,7 @@ void KScreenDaemon::init()
     connect(Device::self(), &Device::lidClosedChanged, this, &KScreenDaemon::lidClosedChanged);
     connect(Device::self(), &Device::resumingFromSuspend, this,
             [&]() {
-                KScreen::Log::instance()->setContext("resuming");
+                KScreen::Log::instance()->setContext(QStringLiteral("resuming"));
                 qCDebug(KSCREEN_KDED) << "Resumed from suspend, checking for screen changes";
                 // We don't care about the result, we just want to force the backend
                 // to query XRandR so that it will detect possible changes that happened
@@ -166,17 +165,32 @@ void KScreenDaemon::applyKnownConfig()
 
     // We may look for a config that has been set when the lid was closed, Bug: 353029
     if (Device::self()->isLaptop() && !Device::self()->isLidClosed()) {
-        Serializer::moveConfig(configId  + QStringLiteral("_lidOpened"), configId);
+        Serializer::moveConfig(configId  + QLatin1String("_lidOpened"), configId);
     }
 
     KScreen::ConfigPtr config = Serializer::config(m_monitoredConfig, configId);
     // It's possible that the Serializer returned a nullptr
     if (!config || !KScreen::Config::canBeApplied(config, KScreen::Config::ValidityFlag::RequireAtLeastOneEnabledScreen)) {
-        return applyIdealConfig();
+        applyIdealConfig();
+        return;
     }
     doApplyConfig(config);
 }
 
+void KScreenDaemon::applyLayoutPreset(const QString &presetName)
+{
+    const QMetaEnum actionEnum = QMetaEnum::fromType<KScreen::OsdAction::Action>();
+    Q_ASSERT(actionEnum.isValid());
+
+    bool ok;
+    auto action = static_cast<KScreen::OsdAction::Action>(actionEnum.keyToValue(qPrintable(presetName), &ok));
+    if (!ok) {
+        qCWarning(KSCREEN_KDED) << "Cannot apply unknown screen layout preset named" << presetName;
+        return;
+    }
+    applyOsdAction(action);
+}
+
 void KScreenDaemon::applyOsdAction(KScreen::OsdAction::Action action)
 {
     switch (action) {
@@ -212,11 +226,11 @@ void KScreenDaemon::applyIdealConfig()
 {
 
     if (m_monitoredConfig->connectedOutputs().count() < 2) {
-        KScreen::OsdManager::self()->hideOsd();
+        m_osdManager->hideOsd();
         doApplyConfig(Generator::self()->idealConfig(m_monitoredConfig));
     } else {
         qCDebug(KSCREEN_KDED) << "Getting ideal config from user via OSD...";
-        auto action = KScreen::OsdManager::self()->showActionSelector();
+        auto action = m_osdManager->showActionSelector();
         connect(action, &KScreen::OsdAction::selected,
                 this, &KScreenDaemon::applyOsdAction);
     }
@@ -247,6 +261,12 @@ void KScreenDaemon::configChanged()
     }
 
     // Reset timer, delay the writeback
+    if (!m_saveTimer) {
+        m_saveTimer = new QTimer(this);
+        m_saveTimer->setInterval(300);
+        m_saveTimer->setSingleShot(true);
+        connect(m_saveTimer, &QTimer::timeout, this, &KScreenDaemon::saveCurrentConfig);
+    }
     m_saveTimer->start();
 }
 
@@ -281,14 +301,14 @@ void KScreenDaemon::showOsd(const QString &icon, const QString &text)
 
 void KScreenDaemon::showOutputIdentifier()
 {
-    KScreen::OsdManager::self()->showOutputIdentifiers();
+    m_osdManager->showOutputIdentifiers();
 }
 
 void KScreenDaemon::displayButton()
 {
     qCDebug(KSCREEN_KDED) << "displayBtn triggered";
 
-    auto action = KScreen::OsdManager::self()->showActionSelector();
+    auto action = m_osdManager->showActionSelector();
     connect(action, &KScreen::OsdAction::selected,
             this, &KScreenDaemon::applyOsdAction);
 }
@@ -314,7 +334,7 @@ void KScreenDaemon::lidClosedChanged(bool lidIsClosed)
         // then the configuration has changed while the lid was closed and we just
         // use applyConfig() and see what we can do ...
 
-        const QString openConfigId = Serializer::configId(m_monitoredConfig) + QStringLiteral("_lidOpened");
+        const QString openConfigId = Serializer::configId(m_monitoredConfig) + QLatin1String("_lidOpened");
         if (Serializer::configExists(openConfigId)) {
             const KScreen::ConfigPtr openedConfig = Serializer::config(m_monitoredConfig, openConfigId);
             Serializer::removeConfig(openConfigId);
@@ -333,10 +353,10 @@ void KScreenDaemon::lidClosedTimeout()
 
     // If we are here, it means that closing the lid did not result in suspend
     // action.
-    // FIXME: This could be simply because the suspend took longer than m_lidClosedTimer
+    // FIXME: This could be because the suspend took longer than m_lidClosedTimer
     // timeout. Ideally we need to be able to look into PowerDevil config to see
     // what's the configured action for lid events, but there's no API to do that
-    // and I'm no parsing PowerDevil's configs...
+    // and I'm not parsing PowerDevil's configs...
 
     qCDebug(KSCREEN_KDED) << "Lid closed without system going to suspend -> turning off the screen";
     for (KScreen::OutputPtr &output : m_monitoredConfig->outputs()) {
@@ -344,7 +364,7 @@ void KScreenDaemon::lidClosedTimeout()
             if (output->isConnected() && output->isEnabled()) {
                 // Save the current config with opened lid, just so that we know
                 // how to restore it later
-                const QString configId = Serializer::configId(m_monitoredConfig) + QStringLiteral("_lidOpened");
+                const QString configId = Serializer::configId(m_monitoredConfig) + QLatin1String("_lidOpened");
                 Serializer::saveConfig(m_monitoredConfig, configId);
                 disableOutput(m_monitoredConfig, output);
                 doApplyConfig(m_monitoredConfig);
@@ -436,16 +456,5 @@ void KScreenDaemon::disableOutput(KScreen::ConfigPtr &config, KScreen::OutputPtr
     output->setEnabled(false);
 }
 
-KScreen::OutputPtr KScreenDaemon::findEmbeddedOutput(const KScreen::ConfigPtr &config)
-{
-    Q_FOREACH (const KScreen::OutputPtr &output, config->outputs()) {
-        if (output->type() == KScreen::Output::Panel) {
-            return output;
-        }
-    }
-
-    return KScreen::OutputPtr();
-}
-
 
 #include "daemon.moc"
